from __future__ import unicode_literals

import contextlib
import hashlib
import os
import re
import tempfile
import shutil
import stat
import zipfile

from base64 import urlsafe_b64encode
from io import StringIO

from poetry.__version__ import __version__
from poetry.semver import parse_constraint
from poetry.utils._compat import Path

from ..utils.helpers import normalize_file_permissions
from ..utils.package_include import PackageInclude
from ..utils.tags import get_abbr_impl
from ..utils.tags import get_abi_tag
from ..utils.tags import get_impl_ver
from ..utils.tags import get_platform
from .builder import Builder


wheel_file_template = """\
Wheel-Version: 1.0
Generator: poetry {version}
Root-Is-Purelib: {pure_lib}
Tag: {tag}
"""


class WheelBuilder(Builder):
    def __init__(self, poetry, env, io, target_dir=None, original=None):
        super(WheelBuilder, self).__init__(poetry, env, io)

        self._records = []
        self._original_path = self._path
        self._target_dir = target_dir or (self._poetry.file.parent / "dist")
        if original:
            self._original_path = original.file.parent

    @classmethod
    def make_in(cls, poetry, env, io, directory=None, original=None):
        wb = WheelBuilder(poetry, env, io, target_dir=directory, original=original)
        wb.build()

        return wb.wheel_filename

    @classmethod
    def make(cls, poetry, env, io):
        """Build a wheel in the dist/ directory, and optionally upload it."""
        cls.make_in(poetry, env, io)

    def build(self):
        self._io.writeln(" - Building <info>wheel</info>")

        dist_dir = self._target_dir
        if not dist_dir.exists():
            dist_dir.mkdir()

        (fd, temp_path) = tempfile.mkstemp(suffix=".whl")

        with zipfile.ZipFile(
            os.fdopen(fd, "w+b"), mode="w", compression=zipfile.ZIP_DEFLATED
        ) as zip_file:
            self._build()
            self._copy_module(zip_file)
            self._write_metadata(zip_file)
            self._write_record(zip_file)

        wheel_path = dist_dir / self.wheel_filename
        if wheel_path.exists():
            wheel_path.unlink()
        shutil.move(temp_path, str(wheel_path))

        self._io.writeln(" - Built <fg=cyan>{}</>".format(self.wheel_filename))

    def _build(self):
        if self._package.build:
            setup = self._path / "setup.py"

            # We need to place ourselves in the temporary
            # directory in order to build the package
            current_path = os.getcwd()
            try:
                os.chdir(str(self._path))
                self._env.run(
                    "python", str(setup), "build", "-b", str(self._path / "build")
                )
            finally:
                os.chdir(current_path)

            build_dir = self._path / "build"
            lib = list(build_dir.glob("lib.*"))
            if not lib:
                # The result of building the extensions
                # does not exist, this may due to conditional
                # builds, so we assume that it's okay
                return

            lib = lib[0]
            for pkg in lib.glob("*"):
                shutil.rmtree(str(self._path / pkg.name))
                shutil.copytree(str(pkg), str(self._path / pkg.name))

    def _copy_module(self, wheel):
        excluded = self.find_excluded_files()
        to_add = []

        for include in self._module.includes:
            include.refresh()

            for file in include.elements:
                if "__pycache__" in str(file):
                    continue

                if file.is_dir():
                    continue

                if isinstance(include, PackageInclude) and include.source:
                    rel_file = file.relative_to(include.base)
                else:
                    rel_file = file.relative_to(self._path)

                if file in excluded:
                    continue

                if file.suffix == ".pyc":
                    continue

                if (file, rel_file) in to_add:
                    # Skip duplicates
                    continue

                self._io.writeln(
                    " - Adding: <comment>{}</comment>".format(str(file)),
                    verbosity=self._io.VERBOSITY_VERY_VERBOSE,
                )
                to_add.append((file, rel_file))

        # Walk the files and compress them,
        # sorting everything so the order is stable.
        for full_path, rel_path in sorted(to_add, key=lambda x: x[1]):
            self._add_file(wheel, full_path, rel_path)

    def _write_metadata(self, wheel):
        if (
            "scripts" in self._poetry.local_config
            or "plugins" in self._poetry.local_config
        ):
            with self._write_to_zip(wheel, self.dist_info + "/entry_points.txt") as f:
                self._write_entry_points(f)

        for base in ("COPYING", "LICENSE"):
            for path in sorted(self._path.glob(base + "*")):
                self._add_file(wheel, path, "%s/%s" % (self.dist_info, path.name))

        with self._write_to_zip(wheel, self.dist_info + "/WHEEL") as f:
            self._write_wheel_file(f)

        with self._write_to_zip(wheel, self.dist_info + "/METADATA") as f:
            self._write_metadata_file(f)

    def _write_record(self, wheel):
        # Write a record of the files in the wheel
        with self._write_to_zip(wheel, self.dist_info + "/RECORD") as f:
            for path, hash, size in self._records:
                f.write("{},sha256={},{}\n".format(path, hash, size))
            # RECORD itself is recorded with no hash or size
            f.write(self.dist_info + "/RECORD,,\n")

    def find_excluded_files(self):  # type: () -> list
        # Checking VCS
        return []

    @property
    def dist_info(self):  # type: () -> str
        return self.dist_info_name(self._package.name, self._meta.version)

    @property
    def wheel_filename(self):  # type: () -> str
        return "{}-{}-{}.whl".format(
            re.sub(r"[^\w\d.]+", "_", self._package.pretty_name, flags=re.UNICODE),
            re.sub(r"[^\w\d.]+", "_", self._meta.version, flags=re.UNICODE),
            self.tag,
        )

    def supports_python2(self):
        return self._package.python_constraint.allows_any(
            parse_constraint(">=2.0.0 <3.0.0")
        )

    def dist_info_name(self, distribution, version):  # type: (...) -> str
        escaped_name = re.sub(r"[^\w\d.]+", "_", distribution, flags=re.UNICODE)
        escaped_version = re.sub(r"[^\w\d.]+", "_", version, flags=re.UNICODE)

        return "{}-{}.dist-info".format(escaped_name, escaped_version)

    @property
    def tag(self):
        if self._package.build:
            platform = get_platform().replace(".", "_").replace("-", "_")
            impl_name = get_abbr_impl(self._env)
            impl_ver = get_impl_ver(self._env)
            impl = impl_name + impl_ver
            abi_tag = str(get_abi_tag(self._env)).lower()
            tag = (impl, abi_tag, platform)
        else:
            platform = "any"
            if self.supports_python2():
                impl = "py2.py3"
            else:
                impl = "py3"

            tag = (impl, "none", platform)

        return "-".join(tag)

    def _add_file(self, wheel, full_path, rel_path):
        full_path, rel_path = str(full_path), str(rel_path)
        if os.sep != "/":
            # We always want to have /-separated paths in the zip file and in
            # RECORD
            rel_path = rel_path.replace(os.sep, "/")

        zinfo = zipfile.ZipInfo(rel_path)

        # Normalize permission bits to either 755 (executable) or 644
        st_mode = os.stat(full_path).st_mode
        new_mode = normalize_file_permissions(st_mode)
        zinfo.external_attr = (new_mode & 0xFFFF) << 16  # Unix attributes

        if stat.S_ISDIR(st_mode):
            zinfo.external_attr |= 0x10  # MS-DOS directory flag

        hashsum = hashlib.sha256()
        with open(full_path, "rb") as src:
            while True:
                buf = src.read(1024 * 8)
                if not buf:
                    break
                hashsum.update(buf)

            src.seek(0)
            wheel.writestr(zinfo, src.read())

        size = os.stat(full_path).st_size
        hash_digest = urlsafe_b64encode(hashsum.digest()).decode("ascii").rstrip("=")

        self._records.append((rel_path, hash_digest, size))

    @contextlib.contextmanager
    def _write_to_zip(self, wheel, rel_path):
        sio = StringIO()
        yield sio

        # The default is a fixed timestamp rather than the current time, so
        # that building a wheel twice on the same computer can automatically
        # give you the exact same result.
        date_time = (2016, 1, 1, 0, 0, 0)
        zi = zipfile.ZipInfo(rel_path, date_time)
        b = sio.getvalue().encode("utf-8")
        hashsum = hashlib.sha256(b)
        hash_digest = urlsafe_b64encode(hashsum.digest()).decode("ascii").rstrip("=")

        wheel.writestr(zi, b, compress_type=zipfile.ZIP_DEFLATED)
        self._records.append((rel_path, hash_digest, len(b)))

    def _write_entry_points(self, fp):
        """
        Write entry_points.txt.
        """
        entry_points = self.convert_entry_points()

        for group_name in sorted(entry_points):
            fp.write("[{}]\n".format(group_name))
            for ep in sorted(entry_points[group_name]):
                fp.write(ep.replace(" ", "") + "\n")

            fp.write("\n")

    def _write_wheel_file(self, fp):
        fp.write(
            wheel_file_template.format(
                version=__version__,
                pure_lib="true" if self._package.build is None else "false",
                tag=self.tag,
            )
        )

    def _write_metadata_file(self, fp):
        """
        Write out metadata in the 2.x format (email like)
        """
        fp.write("Metadata-Version: 2.1\n")
        fp.write("Name: {}\n".format(self._meta.name))
        fp.write("Version: {}\n".format(self._meta.version))
        fp.write("Summary: {}\n".format(self._meta.summary))
        fp.write("Home-page: {}\n".format(self._meta.home_page or "UNKNOWN"))
        fp.write("License: {}\n".format(self._meta.license or "UNKNOWN"))

        # Optional fields
        if self._meta.keywords:
            fp.write("Keywords: {}\n".format(self._meta.keywords))

        if self._meta.author:
            fp.write("Author: {}\n".format(self._meta.author))

        if self._meta.author_email:
            fp.write("Author-email: {}\n".format(self._meta.author_email))

        if self._meta.requires_python:
            fp.write("Requires-Python: {}\n".format(self._meta.requires_python))

        for classifier in self._meta.classifiers:
            fp.write("Classifier: {}\n".format(classifier))

        for extra in sorted(self._meta.provides_extra):
            fp.write("Provides-Extra: {}\n".format(extra))

        for dep in sorted(self._meta.requires_dist):
            fp.write("Requires-Dist: {}\n".format(dep))

        for url in sorted(self._meta.project_urls, key=lambda u: u[0]):
            fp.write("Project-URL: {}\n".format(url))

        if self._meta.description_content_type:
            fp.write(
                "Description-Content-Type: "
                "{}\n".format(self._meta.description_content_type)
            )

        if self._meta.description is not None:
            fp.write("\n" + self._meta.description + "\n")



import re

from typing import List

from ..command import Command


class DebugResolveCommand(Command):
    """
    Debugs dependency resolution.

    debug:resolve
        { package?* : packages to resolve. }
        { --E|extras=* : Extras to activate for the dependency. }
        { --python= : Python version(s) to use for resolution. }
        { --tree : Displays the dependency tree. }
        { --install : Show what would be installed for the current system. }
    """

    _loggers = ["poetry.repositories.pypi_repository"]

    def handle(self):
        from poetry.packages import ProjectPackage
        from poetry.puzzle import Solver
        from poetry.repositories.repository import Repository
        from poetry.semver import parse_constraint
        from poetry.utils.env import Env

        packages = self.argument("package")

        if not packages:
            package = self.poetry.package
        else:
            package = ProjectPackage(
                self.poetry.package.name, self.poetry.package.version
            )
            requirements = self._format_requirements(packages)

            for name, constraint in requirements.items():
                dep = package.add_dependency(name, constraint)
                extras = []
                for extra in self.option("extras"):
                    if " " in extra:
                        extras += [e.strip() for e in extra.split(" ")]
                    else:
                        extras.append(extra)

                for ex in extras:
                    dep.extras.append(ex)

        package.python_versions = self.option("python") or (
            self.poetry.package.python_versions
        )

        pool = self.poetry.pool

        solver = Solver(package, pool, Repository(), Repository(), self.output)

        ops = solver.solve()

        self.line("")
        self.line("Resolution results:")
        self.line("")

        if self.option("tree"):
            show_command = self.get_application().find("show")
            show_command.output = self.output
            show_command.init_styles()

            packages = [op.package for op in ops]
            repo = Repository(packages)

            requires = package.requires + package.dev_requires
            for pkg in repo.packages:
                for require in requires:
                    if pkg.name == require.name:
                        show_command.display_package_tree(pkg, repo)
                        break

            return 0

        env = Env.get()
        current_python_version = parse_constraint(
            ".".join(str(v) for v in env.version_info)
        )
        for op in ops:
            pkg = op.package
            if self.option("install"):
                if not pkg.python_constraint.allows(
                    current_python_version
                ) or not env.is_valid_for_marker(pkg.marker):
                    continue

            self.line(
                "  - <info>{}</info> (<comment>{}</comment>)".format(
                    pkg.name, pkg.version
                )
            )
            if not pkg.python_constraint.is_any():
                self.line("    - python: {}".format(pkg.python_versions))

            if not pkg.marker.is_any():
                self.line("    - marker: {}".format(pkg.marker))

    def _determine_requirements(self, requires):  # type: (List[str]) -> List[str]
        from poetry.semver import parse_constraint

        if not requires:
            return []

        requires = self._parse_name_version_pairs(requires)
        for requirement in requires:
            if "version" in requirement:
                parse_constraint(requirement["version"])

        return requires

    def _parse_name_version_pairs(self, pairs):  # type: (list) -> list
        result = []

        for i in range(len(pairs)):
            if pairs[i].startswith("git+https://"):
                url = pairs[i].lstrip("git+")
                rev = None
                if "@" in url:
                    url, rev = url.split("@")

                pair = {"name": url.split("/")[-1].rstrip(".git"), "git": url}
                if rev:
                    pair["rev"] = rev

                result.append(pair)

                continue

            pair = re.sub("^([^=: ]+)[=: ](.*)$", "\\1 \\2", pairs[i].strip())
            pair = pair.strip()

            if " " in pair:
                name, version = pair.split(" ", 2)
                result.append({"name": name, "version": version})
            else:
                result.append({"name": pair, "version": "*"})

        return result

    def _format_requirements(self, requirements):  # type: (List[str]) -> dict
        requires = {}
        requirements = self._determine_requirements(requirements)

        for requirement in requirements:
            name = requirement.pop("name")
            requires[name] = requirement

        return requires



from .command import Command


class EnvCommand(Command):
    def __init__(self):
        self._env = None

        super(EnvCommand, self).__init__()

    def initialize(self, i, o):
        from poetry.semver import parse_constraint
        from poetry.utils.env import Env

        super(EnvCommand, self).initialize(i, o)

        # Checking compatibility of the current environment with
        # the python dependency specified in pyproject.toml
        current_env = Env.get()
        supported_python = self.poetry.package.python_constraint
        current_python = parse_constraint(
            ".".join(str(v) for v in current_env.version_info[:3])
        )

        if not supported_python.allows(current_python):
            raise RuntimeError(
                "The current Python version ({}) is not supported by the project ({})\n"
                "Please activate a compatible Python version.".format(
                    current_python, self.poetry.package.python_versions
                )
            )

        self._env = Env.create_venv(
            o, self.poetry.package.name, cwd=self.poetry.file.parent
        )

        if self._env.is_venv() and o.is_verbose():
            o.writeln("Using virtualenv: <comment>{}</>".format(self._env.path))

    @property
    def env(self):
        return self._env



import glob
import os
import pkginfo

from pkginfo.distribution import HEADER_ATTRS
from pkginfo.distribution import HEADER_ATTRS_2_0

from poetry.io import NullIO
from poetry.utils._compat import PY35
from poetry.utils._compat import Path
from poetry.utils.helpers import parse_requires
from poetry.utils.toml_file import TomlFile
from poetry.utils.env import Env
from poetry.utils.env import EnvCommandError
from poetry.utils.setup_reader import SetupReader

from .dependency import Dependency

# Patching pkginfo to support Metadata version 2.1 (PEP 566)
HEADER_ATTRS.update(
    {"2.1": HEADER_ATTRS_2_0 + (("Provides-Extra", "provides_extra", True),)}
)


class DirectoryDependency(Dependency):
    def __init__(
        self,
        path,  # type: Path
        category="main",  # type: str
        optional=False,  # type: bool
        base=None,  # type: Path
        develop=True,  # type: bool
    ):
        from . import dependency_from_pep_508
        from .package import Package

        self._path = path
        self._base = base
        self._full_path = path
        self._develop = develop
        self._supports_poetry = False

        if self._base and not self._path.is_absolute():
            self._full_path = self._base / self._path

        if not self._full_path.exists():
            raise ValueError("Directory {} does not exist".format(self._path))

        if self._full_path.is_file():
            raise ValueError("{} is a file, expected a directory".format(self._path))

        # Checking content to dertermine actions
        setup = self._full_path / "setup.py"
        pyproject = TomlFile(self._full_path / "pyproject.toml")
        if pyproject.exists():
            pyproject_content = pyproject.read()
            self._supports_poetry = (
                "tool" in pyproject_content and "poetry" in pyproject_content["tool"]
            )

        if not setup.exists() and not self._supports_poetry:
            raise ValueError(
                "Directory {} does not seem to be a Python package".format(
                    self._full_path
                )
            )

        if self._supports_poetry:
            from poetry.poetry import Poetry

            poetry = Poetry.create(self._full_path)

            package = poetry.package
            self._package = Package(package.pretty_name, package.version)
            self._package.requires += package.requires
            self._package.dev_requires += package.dev_requires
            self._package.extras = package.extras
            self._package.python_versions = package.python_versions
        else:
            # Execute egg_info
            current_dir = os.getcwd()
            os.chdir(str(self._full_path))

            try:
                cwd = base
                venv = Env.get(NullIO(), cwd=cwd)
                venv.run("python", "setup.py", "egg_info")
            except EnvCommandError:
                result = SetupReader.read_from_directory(self._full_path)
                if not result["name"]:
                    # The name could not be determined
                    # so we raise an error since it is mandatory
                    raise RuntimeError(
                        "Unable to retrieve the package name for {}".format(path)
                    )

                if not result["version"]:
                    # The version could not be determined
                    # so we raise an error since it is mandatory
                    raise RuntimeError(
                        "Unable to retrieve the package version for {}".format(path)
                    )

                package_name = result["name"]
                package_version = result["version"]
                python_requires = result["python_requires"]
                if python_requires is None:
                    python_requires = "*"

                package_summary = ""

                requires = ""
                for dep in result["install_requires"]:
                    requires += dep + "\n"

                if result["extras_require"]:
                    requires += "\n"

                for extra_name, deps in result["extras_require"].items():
                    requires += "[{}]\n".format(extra_name)

                    for dep in deps:
                        requires += dep + "\n"

                    requires += "\n"

                reqs = parse_requires(requires)
            else:
                os.chdir(current_dir)
                # Sometimes pathlib will fail on recursive
                # symbolic links, so we need to workaround it
                # and use the glob module instead.
                # Note that this does not happen with pathlib2
                # so it's safe to use it for Python < 3.4.
                if PY35:
                    egg_info = next(
                        Path(p)
                        for p in glob.glob(
                            os.path.join(str(self._full_path), "**", "*.egg-info"),
                            recursive=True,
                        )
                    )
                else:
                    egg_info = next(self._full_path.glob("**/*.egg-info"))

                meta = pkginfo.UnpackedSDist(str(egg_info))
                package_name = meta.name
                package_version = meta.version
                package_summary = meta.summary
                python_requires = meta.requires_python

                if meta.requires_dist:
                    reqs = list(meta.requires_dist)
                else:
                    reqs = []
                    requires = egg_info / "requires.txt"
                    if requires.exists():
                        with requires.open() as f:
                            reqs = parse_requires(f.read())
            finally:
                os.chdir(current_dir)

            package = Package(package_name, package_version)
            package.description = package_summary

            for req in reqs:
                package.requires.append(dependency_from_pep_508(req))

            if python_requires:
                package.python_versions = python_requires

            self._package = package

        self._package.source_type = "directory"
        self._package.source_url = self._path.as_posix()

        super(DirectoryDependency, self).__init__(
            self._package.name,
            self._package.version,
            category=category,
            optional=optional,
            allows_prereleases=True,
        )

    @property
    def path(self):
        return self._path

    @property
    def full_path(self):
        return self._full_path.resolve()

    @property
    def package(self):
        return self._package

    @property
    def develop(self):
        return self._develop

    def supports_poetry(self):
        return self._supports_poetry

    def is_directory(self):
        return True



from cleo.inputs import ListInput
from cleo.outputs import NullOutput

from poetry.console.styles.poetry import PoetryStyle


class NullIO(PoetryStyle):
    def __init__(self):
        super(NullIO, self).__init__(ListInput([]), NullOutput())

    def is_quiet(self):  # type: () -> bool
        return False

    def is_verbose(self):  # type: () -> bool
        return False

    def is_very_verbose(self):  # type: () -> bool
        return False

    def is_debug(self):  # type: () -> bool
        return False

    def writeln(self, *args, **kwargs):
        pass

    def write(self, *args, **kwargs):
        pass

    def new_line(self, *args, **kwargs):
        pass



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['poetry',
 'poetry.console',
 'poetry.console.commands',
 'poetry.console.commands.cache',
 'poetry.console.commands.debug',
 'poetry.console.commands.self',
 'poetry.console.styles',
 'poetry.installation',
 'poetry.io',
 'poetry.json',
 'poetry.layouts',
 'poetry.masonry',
 'poetry.masonry.builders',
 'poetry.masonry.publishing',
 'poetry.masonry.utils',
 'poetry.mixology',
 'poetry.packages',
 'poetry.packages.constraints',
 'poetry.packages.utils',
 'poetry.puzzle',
 'poetry.puzzle.operations',
 'poetry.repositories',
 'poetry.semver',
 'poetry.spdx',
 'poetry.utils',
 'poetry.vcs',
 'poetry.version']

package_data = \
{'': ['*'],
 'poetry': ['_vendor/*'],
 'poetry.json': ['schemas/*'],
 'poetry.spdx': ['data/*']}

install_requires = \
['cachecontrol[filecache]>=0.12.4,<0.13.0',
 'cachy>=0.2,<0.3',
 'cleo>=0.6.7,<0.7.0',
 'html5lib>=1.0,<2.0',
 'jsonschema>=3.0a3,<4.0',
 'pkginfo>=1.4,<2.0',
 'pyparsing>=2.2,<3.0',
 'pyrsistent>=0.14.2,<0.15.0',
 'requests-toolbelt>=0.8.0,<0.9.0',
 'requests>=2.18,<3.0',
 'shellingham>=1.1,<2.0',
 'tomlkit>=0.4.4,<0.5.0']

extras_require = \
{':python_version >= "2.7" and python_version < "2.8"': ['virtualenv>=16.0,<17.0'],
 ':python_version >= "2.7" and python_version < "2.8" or python_version >= "3.4" and python_version < "3.5"': ['typing>=3.6,<4.0',
                                                                                                               'pathlib2>=2.3,<3.0']}

entry_points = \
{'console_scripts': ['poetry = poetry.console:main']}

setup_kwargs = {
    'name': 'poetry',
    'version': '0.12.6',
    'description': 'Python dependency management and packaging made easy.',
    'long_description': '# Poetry: Dependency Management for Python\n\n![Poetry build status](https://travis-ci.org/sdispater/poetry.svg)\n\nPoetry helps you declare, manage and install dependencies of Python projects,\nensuring you have the right stack everywhere.\n\n![Poetry Install](https://raw.githubusercontent.com/sdispater/poetry/master/assets/install.gif)\n\nIt supports Python 2.7 and 3.4+.\n\n## Installation\n\nPoetry provides a custom installer that will install `poetry` isolated\nfrom the rest of your system by vendorizing its dependencies. This is the\nrecommended way of installing `poetry`.\n\n```bash\ncurl -sSL https://raw.githubusercontent.com/sdispater/poetry/master/get-poetry.py | python\n```\n\nAlternatively, you can download the `get-poetry.py` file and execute it separately.\n\nIf you want to install prerelease versions, you can do so by passing `--preview` to `get-poetry.py`:\n\n```bash\npython get-poetry.py --preview\n```\n\nSimilarly, if you want to install a specific version, you can use `--version`:\n\n```bash\npython get-poetry.py --version 0.7.0\n```\n\nUsing `pip` to install `poetry` is also possible.\n\n```bash\npip install --user poetry\n```\n\nBe aware, however, that it will also install poetry\'s dependencies\nwhich might cause conflicts.\n\n## Updating `poetry`\n\nUpdating poetry to the latest stable version is as simple as calling the `self:update` command.\n\n```bash\npoetry self:update\n```\n\nIf you want to install prerelease versions, you can use the `--preview` option.\n\n```bash\npoetry self:update --preview\n```\n\nAnd finally, if you want to install a specific version you can pass it as an argument\nto `self:update`.\n\n```bash\npoetry self:update 0.8.0\n```\n\n\n## Enable tab completion for Bash, Fish, or Zsh\n\n`poetry` supports generating completion scripts for Bash, Fish, and Zsh.\nSee `poetry help completions` for full details, but the gist is as simple as using one of the following:\n\n```bash\n# Bash\npoetry completions bash > /etc/bash_completion.d/poetry.bash-completion\n\n# Bash (macOS/Homebrew)\npoetry completions bash > $(brew --prefix)/etc/bash_completion.d/poetry.bash-completion\n\n# Fish\npoetry completions fish > ~/.config/fish/completions/poetry.fish\n\n# Zsh\npoetry completions zsh > ~/.zfunc/_poetry\n```\n\n*Note:* you may need to restart your shell in order for the changes to take\neffect.\n\nFor `zsh`, you must then add the following line in your `~/.zshrc` before\n`compinit`:\n\n```zsh\nfpath+=~/.zfunc\n```\n\n\n## Introduction\n\n`poetry` is a tool to handle dependency installation as well as building and packaging of Python packages.\nIt only needs one file to do all of that: the new, [standardized](https://www.python.org/dev/peps/pep-0518/) `pyproject.toml`.\n\nIn other words, poetry uses `pyproject.toml` to replace `setup.py`, `requirements.txt`, `setup.cfg`, `MANIFEST.in` and the newly added `Pipfile`.\n\n```toml\n[tool.poetry]\nname = "my-package"\nversion = "0.1.0"\ndescription = "The description of the package"\n\nlicense = "MIT"\n\nauthors = [\n    "Sébastien Eustace <sebastien@eustace.io>"\n]\n\nreadme = \'README.md\'  # Markdown files are supported\n\nrepository = "https://github.com/sdispater/poetry"\nhomepage = "https://github.com/sdispater/poetry"\n\nkeywords = [\'packaging\', \'poetry\']\n\n[tool.poetry.dependencies]\npython = "~2.7 || ^3.2"  # Compatible python versions must be declared here\ntoml = "^0.9"\n# Dependencies with extras\nrequests = { version = "^2.13", extras = [ "security" ] }\n# Python specific dependencies with prereleases allowed\npathlib2 = { version = "^2.2", python = "~2.7", allows-prereleases = true }\n# Git dependencies\ncleo = { git = "https://github.com/sdispater/cleo.git", branch = "master" }\n\n# Optional dependencies (extras)\npendulum = { version = "^1.4", optional = true }\n\n[tool.poetry.dev-dependencies]\npytest = "^3.0"\npytest-cov = "^2.4"\n\n[tool.poetry.scripts]\nmy-script = \'my_package:main\'\n```\n\nThere are some things we can notice here:\n\n* It will try to enforce [semantic versioning](<http://semver.org>) as the best practice in version naming.\n* You can specify the readme, included and excluded files: no more `MANIFEST.in`.\n`poetry` will also use VCS ignore files (like `.gitignore`) to populate the `exclude` section.\n* Keywords (up to 5) can be specified and will act as tags on the packaging site.\n* The dependencies sections support caret, tilde, wildcard, inequality and multiple requirements.\n* You must specify the python versions for which your package is compatible.\n\n`poetry` will also detect if you are inside a virtualenv and install the packages accordingly.\nSo, `poetry` can be installed globally and used everywhere.\n\n`poetry` also comes with a full fledged dependency resolution library.\n\n## Why?\n\nPackaging systems and dependency management in Python are rather convoluted and hard to understand for newcomers.\nEven for seasoned developers it might be cumbersome at times to create all files needed in a Python project: `setup.py`,\n`requirements.txt`, `setup.cfg`, `MANIFEST.in` and the newly added `Pipfile`.\n\nSo I wanted a tool that would limit everything to a single configuration file to do:\ndependency management, packaging and publishing.\n\nIt takes inspiration in tools that exist in other languages, like `composer` (PHP) or `cargo` (Rust).\n\nAnd, finally, there is no reliable tool to properly resolve dependencies in Python, so I started `poetry`\nto bring an exhaustive dependency resolver to the Python community.\n\n### What about Pipenv?\n\nIn short: I do not like the CLI it provides, or some of the decisions made,\nand I think we can make a better and more intuitive one. Here are a few things\nthat I don\'t like.\n\n#### Dependency resolution\n\nThe dependency resolution is erratic and will fail even if there is a solution. Let\'s take an example:\n\n```bash\npipenv install oslo.utils==1.4.0\n```\n\nwill fail with this error:\n\n```text\nCould not find a version that matches pbr!=0.7,!=2.1.0,<1.0,>=0.6,>=2.0.0\n```\n\nwhile Poetry will get you the right set of packages:\n\n```bash\npoetry add oslo.utils=1.4.0\n```\n\nresults in :\n\n```text\n  - Installing pytz (2018.3)\n  - Installing netifaces (0.10.6)\n  - Installing netaddr (0.7.19)\n  - Installing oslo.i18n (2.1.0)\n  - Installing iso8601 (0.1.12)\n  - Installing six (1.11.0)\n  - Installing babel (2.5.3)\n  - Installing pbr (0.11.1)\n  - Installing oslo.utils (1.4.0)\n```\n\nThis is possible thanks to the efficient dependency resolver at the heart of Poetry.\n\nHere is a breakdown of what exactly happens here:\n\n`oslo.utils (1.4.0)` depends on:\n\n- `pbr (>=0.6,!=0.7,<1.0)`\n- `Babel (>=1.3)`\n- `six (>=1.9.0)`\n- `iso8601 (>=0.1.9)`\n- `oslo.i18n (>=1.3.0)`\n- `netaddr (>=0.7.12)`\n- `netifaces (>=0.10.4)`\n\nWhat interests us is `pbr (>=0.6,!=0.7,<1.0)`.\n\nAt this point, poetry will choose `pbr==0.11.1` which is the latest version that matches the constraint.\n\nNext it will try to select `oslo.i18n==3.20.0` which is the latest version that matches `oslo.i18n (>=1.3.0)`.\n\nHowever this version requires `pbr (!=2.1.0,>=2.0.0)` which is incompatible with `pbr==0.11.1`,\nso `poetry` will try to find a version of `oslo.i18n` that satisfies `pbr (>=0.6,!=0.7,<1.0)`.\n\nBy analyzing the releases of `oslo.i18n`, it will find `oslo.i18n==2.1.0` which requires `pbr (>=0.11,<2.0)`.\nAt this point the rest of the resolution is straightforward since there is no more conflict.\n\n#### Install command\n\nWhen you specify a package to the `install` command it will add it as a wildcard\ndependency. This means that **any** version of this package can be installed which\ncan lead to compatibility issues.\n\nAlso, you have to explicitly tell it to not update the locked packages when you\ninstall new ones. This should be the default.\n\n#### Remove command\n\nThe `remove` command will only remove the package specified but not its dependencies\nif they are no longer needed.\n\nYou either have to use `sync` or `clean` to fix that.\n\n#### Too limited in scope\n\nFinally, the `Pipfile` is just a replacement from `requirements.txt` and, in the end, you will still need to\npopulate your `setup.py` file (or `setup.cfg`) with the exact same dependencies you declared in your `Pipfile`.\nSo, in the end, you will still need to manage a few configuration files to properly setup your project.\n\n\n## Commands\n\n\n### new\n\nThis command will help you kickstart your new Python project by creating\na directory structure suitable for most projects.\n\n```bash\npoetry new my-package\n```\n\nwill create a folder as follows:\n\n```text\nmy-package\n├── pyproject.toml\n├── README.rst\n├── my_package\n│   └── __init__.py\n└── tests\n    ├── __init__.py\n    └── test_my_package\n```\n\nIf you want to name your project differently than the folder, you can pass\nthe `--name` option:\n\n```bash\npoetry new my-folder --name my-package\n```\n\n### init\n\nThis command will help you create a `pyproject.toml` file interactively\nby prompting you to provide basic information about your package.\n\nIt will interactively ask you to fill in the fields, while using some smart defaults.\n\n```bash\npoetry init\n```\n\n#### Options\n\n* `--name`: Name of the package.\n* `--description`: Description of the package.\n* `--author`: Author of the package.\n* `--dependency`: Package to require with a version constraint. Should be in format `foo:1.0.0`.\n* `--dev-dependency`: Development requirements, see `--require`.\n\n### install\n\nThe `install` command reads the `pyproject.toml` file from the current directory, resolves the dependencies,\nand installs them.\n\n```bash\npoetry install\n```\n\nIf there is a `poetry.lock` file in the current directory,\nit will use the exact versions from there instead of resolving them.\nThis ensures that everyone using the library will get the same versions of the dependencies.\n\nIf there is no `poetry.lock` file, Poetry will create one after dependency resolution.\n\nYou can specify to the command that you do not want the development dependencies installed by passing\nthe `--no-dev` option.\n\n```bash\npoetry install --no-dev\n```\n\nYou can also specify the extras you want installed\nby passing the `--E|--extras` option (See [Extras](#extras) for more info)\n\n```bash\npoetry install --extras "mysql pgsql"\npoetry install -E mysql -E pgsql\n```\n\n#### Options\n\n* `--no-dev`: Do not install dev dependencies.\n* `-E|--extras`: Features to install (multiple values allowed).\n\n### update\n\nIn order to get the latest versions of the dependencies and to update the `poetry.lock` file,\nyou should use the `update` command.\n\n```bash\npoetry update\n```\n\nThis will resolve all dependencies of the project and write the exact versions into `poetry.lock`.\n\nIf you just want to update a few packages and not all, you can list them as such:\n\n```bash\npoetry update requests toml\n```\n\n#### Options\n\n* `--dry-run` : Outputs the operations but will not execute anything (implicitly enables --verbose).\n* `--no-dev` : Do not install dev dependencies.\n* `--lock` : Do not perform install (only update the lockfile).\n\n### add\n\nThe `add` command adds required packages to your `pyproject.toml` and installs them.\n\nIf you do not specify a version constraint,\npoetry will choose a suitable one based on the available package versions.\n\n```bash\npoetry add requests pendulum\n```\n\n#### Options\n\n* `--D|dev`: Add package as development dependency.\n* `--optional` : Add as an optional dependency.\n* `--dry-run` : Outputs the operations but will not execute anything (implicitly enables --verbose).\n\n\n### remove\n\nThe `remove` command removes a package from the current\nlist of installed packages\n\n```bash\npoetry remove pendulum\n```\n\n#### Options\n\n* `--D|dev`: Removes a package from the development dependencies.\n* `--dry-run` : Outputs the operations but will not execute anything (implicitly enables --verbose).\n\n\n### show\n\nTo list all of the available packages, you can use the `show` command.\n\n```bash\npoetry show\n```\n\nIf you want to see the details of a certain package, you can pass the package name.\n\n```bash\npoetry show pendulum\n\nname        : pendulum\nversion     : 1.4.2\ndescription : Python datetimes made easy\n\ndependencies:\n - python-dateutil >=2.6.1\n - tzlocal >=1.4\n - pytzdata >=2017.2.2\n```\n\n#### Options\n\n* `--no-dev`: Do not list the dev dependencies.\n* `--tree`: List the dependencies as a tree.\n* `-l|--latest`: Show the latest version.\n*  `-o|--outdated`: Show the latest version but only for packages that are outdated.\n\n\n### build\n\nThe `build` command builds the source and wheels archives.\n\n```bash\npoetry build\n```\n\nNote that, at the moment, only pure python wheels are supported.\n\n#### Options\n\n* `-F|--format`: Limit the format to either wheel or sdist.\n\n### publish\n\nThis command builds (if not already built) and publishes the package to the remote repository.\n\nIt will automatically register the package before uploading if this is the first time it is submitted.\n\n```bash\npoetry publish\n```\n\n#### Options\n\n* `-r|--repository`: The repository to register the package to (default: `pypi`).\nShould match a repository name set by the [`config`](#config) command.\n* `--username (-u)`: The username to access the repository.\n* `--password (-p)`: The password to access the repository.\n\n\n### `config`\n\nThe `config` command allows you to edit poetry config settings and repositories.\n\n```bash\npoetry config --list\n```\n\n#### Usage\n\n````bash\npoetry config [options] [setting-key] [setting-value1] ... [setting-valueN]\n````\n\n`setting-key` is a configuration option name and `setting-value1` is a configuration value.\n\n#### Modifying repositories\n\nIn addition to modifying the config section,\nthe config command also supports making changes to the repositories section by using it the following way:\n\n```bash\npoetry config repositories.foo https://foo.bar/simple/\n```\n\nThis will set the url for repository `foo` to `https://foo.bar/simple/`.\n\nIf you want to store your credentials for a specific repository, you can do so easily:\n\n```bash\npoetry config http-basic.foo username password\n```\n\nIf you do not specify the password you will be prompted to write it.\n\n#### Options\n\n* `--unset`: Remove the configuration element named by `setting-key`.\n* `--list`: Show the list of current config variables.\n\n### search\n\nThis command searches for packages on a remote index.\n\n```bash\npoetry search requests pendulum\n```\n\n#### Options\n\n* `-N|--only-name`: Search only in name.\n\n### lock\n\nThis command locks (without installing) the dependencies specified in `pyproject.toml`.\n\n```bash\npoetry lock\n```\n\n\n## The `pyproject.toml` file\n\nThe `tool.poetry` section of the `pyproject.toml` file is composed of multiple sections.\n\n#### name\n\nThe name of the package. **Required**\n\n#### version\n\nThe version of the package. **Required**\n\nThis should follow [semantic versioning](http://semver.org/). However it will not be enforced and you remain\nfree to follow another specification.\n\n#### description\n\nA short description of the package. **Required**\n\n#### license\n\nThe license of the package.\n\nThe recommended notation for the most common licenses is (alphabetical):\n\n* Apache-2.0\n* BSD-2-Clause\n* BSD-3-Clause\n* BSD-4-Clause\n* GPL-2.0\n* GPL-2.0+\n* GPL-3.0\n* GPL-3.0+\n* LGPL-2.1\n* LGPL-2.1+\n* LGPL-3.0\n* LGPL-3.0+\n* MIT\n\nOptional, but it is highly recommended to supply this.\nMore identifiers are listed at the [SPDX Open Source License Registry](https://www.spdx.org/licenses/).\n\n#### authors\n\nThe authors of the package. This is a list of authors and should contain at least one author.\n\nAuthors must be in the form `name <email>`.\n\n#### readme\n\nThe readme file of the package. **Required**\n\nThe file can be either `README.rst` or `README.md`.\n\n#### homepage\n\nAn URL to the website of the project. **Optional**\n\n#### repository\n\nAn URL to the repository of the project. **Optional**\n\n#### documentation\n\nAn URL to the documentation of the project. **Optional**\n\n#### keywords\n\nA list of keywords (max: 5) that the package is related to. **Optional**\n\n#### include and exclude\n\nA list of patterns that will be included in the final package.\n\nYou can explicitly specify to Poetry that a set of globs should be ignored or included for the purposes of packaging.\nThe globs specified in the exclude field identify a set of files that are not included when a package is built.\n\nIf a VCS is being used for a package, the exclude field will be seeded with the VCS’ ignore settings (`.gitignore` for git for example).\n\n```toml\n[tool.poetry]\n# ...\ninclude = ["package/**/*.py", "package/**/.c"]\n```\n\n```toml\nexclude = ["package/excluded.py"]\n```\n\n### `dependencies` and `dev-dependencies`\n\nPoetry is configured to look for dependencies on [PyPi](https://pypi.org) by default.\nOnly the name and a version string are required in this case.\n\n```toml\n[tool.poetry.dependencies]\nrequests = "^2.13.0"\n```\n\nIf you want to use a private repository, you can add it to your `pyproject.toml` file, like so:\n\n```toml\n[[tool.poetry.source]]\nname = \'private\'\nurl = \'http://example.com/simple\'\n```\n\nBe aware that declaring the python version for which your package\nis compatible is mandatory:\n\n```toml\n[tool.poetry.dependencies]\npython = "^3.6"\n```\n\n#### Caret requirement\n\n**Caret requirements** allow SemVer compatible updates to a specified version.\nAn update is allowed if the new version number does not modify the left-most non-zero digit in the major, minor, patch grouping.\nIn this case, if we ran `poetry update requests`, poetry would update us to version `2.14.0` if it was available,\nbut would not update us to `3.0.0`.\nIf instead we had specified the version string as `^0.1.13`, poetry would update to `0.1.14` but not `0.2.0`.\n`0.0.x` is not considered compatible with any other version.\n\nHere are some more examples of caret requirements and the versions that would be allowed with them:\n\n```text\n^1.2.3 := >=1.2.3 <2.0.0\n^1.2 := >=1.2.0 <2.0.0\n^1 := >=1.0.0 <2.0.0\n^0.2.3 := >=0.2.3 <0.3.0\n^0.0.3 := >=0.0.3 <0.0.4\n^0.0 := >=0.0.0 <0.1.0\n^0 := >=0.0.0 <1.0.0\n```\n\n#### Tilde requirements\n\n**Tilde requirements** specify a minimal version with some ability to update.\nIf you specify a major, minor, and patch version or only a major and minor version, only patch-level changes are allowed.\nIf you only specify a major version, then minor- and patch-level changes are allowed.\n\n`~1.2.3` is an example of a tilde requirement.\n\n```text\n~1.2.3 := >=1.2.3 <1.3.0\n~1.2 := >=1.2.0 <1.3.0\n~1 := >=1.0.0 <2.0.0\n```\n\n#### Wildcard requirements\n\n**Wildcard requirements** allow for any version where the wildcard is positioned.\n\n`*`, `1.*` and `1.2.*` are examples of wildcard requirements.\n\n```text\n* := >=0.0.0\n1.* := >=1.0.0 <2.0.0\n1.2.* := >=1.2.0 <1.3.0\n```\n\n#### Inequality requirements\n\n**Inequality requirements** allow manually specifying a version range or an exact version to depend on.\n\nHere are some examples of inequality requirements:\n\n```text\n>= 1.2.0\n> 1\n< 2\n!= 1.2.3\n```\n\n#### Multiple requirements\n\nMultiple version requirements can also be separated with a comma, e.g. `>= 1.2, < 1.5`.\n\n#### `git` dependencies\n\nTo depend on a library located in a `git` repository,\nthe minimum information you need to specify is the location of the repository with the git key:\n\n```toml\n[tool.poetry.dependencies]\nrequests = { git = "https://github.com/requests/requests.git" }\n```\n\nSince we haven’t specified any other information,\nPoetry assumes that we intend to use the latest commit on the `master` branch to build our project.\nYou can combine the `git` key with the `rev`, `tag`, or `branch` keys to specify something else.\nHere\'s an example of specifying that you want to use the latest commit on a branch named `next`:\n\n```toml\n[tool.poetry.dependencies]\nrequests = { git = "https://github.com/kennethreitz/requests.git", branch = "next" }\n```\n\n#### Python restricted dependencies\n\nYou can also specify that a dependency should be installed only for specific Python versions:\n\n```toml\n[tool.poetry.dependencies]\npathlib2 = { version = "^2.2", python = "~2.7" }\n```\n\n```toml\n[tool.poetry.dependencies]\npathlib2 = { version = "^2.2", python = ["~2.7", "^3.2"] }\n```\n\n### `scripts`\n\nThis section describe the scripts or executable that will be installed when installing the package\n\n```toml\n[tool.poetry.scripts]\npoetry = \'poetry.console:run\'\n```\n\nAfter installing a package with the above toml, `poetry` will be a global command available from the command line that will execute `console.run` in the `poetry` package.\n\n### `extras`\n\nPoetry supports extras to allow expression of:\n\n* optional dependencies, which enhance a package, but are not required; and\n* clusters of optional dependencies.\n\n```toml\n[tool.poetry]\nname = "awesome"\n\n[tool.poetry.dependencies]\n# These packages are mandatory and form the core of this package’s distribution.\nmandatory = "^1.0"\n\n# A list of all of the optional dependencies, some of which are included in the\n# below `extras`. They can be opted into by apps.\npsycopg2 = { version = "^2.7", optional = true }\nmysqlclient = { version = "^1.3", optional = true }\n\n[tool.poetry.extras]\nmysql = ["mysqlclient"]\npgsql = ["psycopg2"]\n```\n\nWhen installing packages, you can specify extras by using the `-E|--extras` option:\n\n```bash\npoetry install --extras "mysql pgsql"\npoetry install -E mysql -E pgsql\n```\n\n### `plugins`\n\nPoetry supports arbitrary plugins which work similarly to\n[setuptools entry points](http://setuptools.readthedocs.io/en/latest/setuptools.html).\nTo match the example in the setuptools documentation, you would use the following:\n\n```toml\n[tool.poetry.plugins] # Optional super table\n\n[tool.poetry.plugins."blogtool.parsers"]\n".rst" = "some_module::SomeClass"\n```\n\n## Resources\n\n* [Official Website](https://poetry.eustace.io)\n* [Issue Tracker](https://github.com/sdispater/poetry/issues)\n',
    'author': 'Sébastien Eustace',
    'author_email': 'sebastien@eustace.io',
    'url': 'https://poetry.eustace.io/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
}


setup(**setup_kwargs)
